/*
 * pc300cfg.c	Cyclades-PC300(tm) configuration utility.
 *
 * Author:	Ivan Passos <ivan.passos@cyclades.com>
 *
 * Copyright:	(c) 1999-2004 Cyclades Corp.
 *
 *      This program is free software; you can redistribute it and/or
 *      modify it under the terms of the GNU General Public License
 *      as published by the Free Software Foundation; either version
 *      2 of the License, or (at your option) any later version.
 *
 */

#ifndef __attribute_const__
#define __attribute_const__
#endif

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <netinet/in.h>
#include <linux/netdevice.h>
#include <linux/if.h>
#include <stdio.h>
#include <errno.h>
#include <fcntl.h>
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <netdb.h>

#include "pc300-types.h"
#include "pc300.h"

#define	CMDSZ	10

int skfd = -1;

static void
pc300_show_if(char *cpc_name, pc300chconf_t *conf)
{
    printf("Interface: %s\n", cpc_name);
    printf("- Media:\t\t");
    switch(conf->media) {
	case IF_IFACE_V24:
	    printf("RS232\n");
	    break;
	case IF_IFACE_V35:
	    printf("V.35\n");
	    break;
	case IF_IFACE_X21:
	    printf("X.21\n");
	    break;
	case IF_IFACE_T1:
	case IF_IFACE_E1:
	    if (conf->media == IF_IFACE_T1) {
		printf("T1\n");
		printf("  - Line Build Out:\t");
		switch(conf->lbo) {
		    case PC300_LBO_0_DB:
			printf("0 dB\n");
			break;
		    case PC300_LBO_7_5_DB:
			printf("7.5 dB\n");
			break;
		    case PC300_LBO_15_DB:
			printf("15 dB\n");
			break;
		    case PC300_LBO_22_5_DB:
			printf("22.5 dB\n");
			break;
		    default:
			printf("unknown\n");
			break;
		}
	    } else {
		printf("E1\n");
	    }
	    printf("  - Line Code:\t\t");
	    switch(conf->lcode) {
		case PC300_LC_AMI:
		    printf("AMI\n");
		    break;
		case PC300_LC_B8ZS:
		    printf("B8ZS\n");
		    break;
		case PC300_LC_NRZ:
		    printf("NRZ\n");
		    break;
		case PC300_LC_HDB3:
		    printf("HDB3\n");
		    break;
		default:
		    printf("unknown\n");
		    break;
	    }
	    printf("  - Frame Mode:\t\t");
	    switch (conf->fr_mode) {
		case PC300_FR_D4:
		    printf("D4\n");
		    break;
		case PC300_FR_ESF:
		    printf("ESF\n");
		    break;
		case PC300_FR_MF_CRC4:
		    printf("CRC4\n");
		    break;
		case PC300_FR_MF_NON_CRC4:
		    printf("Non-CRC4\n");
		    break;
		case PC300_FR_UNFRAMED:
		    printf("Unframed\n");
		    break;
		default:
		    printf("unknown\n");
		    break;
	    }
	    printf("  - RX Sensitivity:\t");
	    switch(conf->rx_sens) {
		case PC300_RX_SENS_SH:
		    printf("Short Haul\n");
		    break;
		case PC300_RX_SENS_LH:
		    printf("Long Haul\n");
		    break;
		default:
		    printf("unknown\n");
		    break;
	    }
	    printf("  - Active channels:\t");
	    if(conf->tslot_bitmap == 0xffffffff) {
		printf("all\n");
	    } else {
		int i, j;
		int ch_range = ((conf->media == IF_IFACE_T1) ? 24 : 32);

		for (i = 0, j = 0; i < ch_range ; i++) {
		    if (conf->tslot_bitmap & (1<<i)) {
			printf("%d, ", i+1);
			j++;
		    }
		}
		printf("\b\b (%d channels)\n", j);
	    }
	    break;
	default:
	    printf("unknown\n");
	    break;
    }

    printf("- Protocol:\t\t");
    switch(conf->proto) {
	case IF_PROTO_X25:
	    printf("X.25\n");
	    break;
	case IF_PROTO_PPP:
	    printf("PPP\n");
	    break;
	case IF_PROTO_CISCO:
	    printf("Cisco HDLC\n");
	    break;
	case IF_PROTO_FR:
	    printf("Frame Relay\n");
	    break;
/*	case (IF_PROTO_FR_ANSI|IF_PROTO_DCE):
	    printf("Frame Relay (ANSI, DCE)\n");
	    break;
	case (IF_PROTO_FR_CCITT|IF_PROTO_DCE):
	    printf("Frame Relay (CCITT, DCE)\n");
	    break;
*/	case IF_PROTO_HDLC:
	    if (conf->monitor) {
		printf("Monitor Mode\n");
	    } else {
		printf("Raw HDLC\n");
		printf("- Encoding:\t\t");
		switch (conf->proto_settings.encoding) {
		    case ENCODING_NRZ:
			printf("NRZ\n");
			break;
		    case ENCODING_NRZI:
			printf("NRZI\n");
			break;
		    case ENCODING_FM_MARK:
			printf("FM1 (Frequency modulation mark)\n");
			break;
		    case ENCODING_FM_SPACE:
			printf("FM0 (Frequency modulation space)\n");
			break;
		    case ENCODING_MANCHESTER:
			printf("Manchester\n");
			break;
		    default:
			printf("unknown\n");
			break;
		}
		printf("- CRC Calculation:\t");
		switch (conf->proto_settings.parity) {
		    case PARITY_NONE:
			printf("None\n");
			break;
		    case PARITY_CRC16_PR0:
			printf("CRC-16 preset 0\n");
			break;
		    case PARITY_CRC16_PR1:
			printf("CRC-16 preset 1\n");
			break;
		    case PARITY_CRC16_PR1_CCITT:
			printf("16-bit CRC-CCITT preset 1\n");
			break;
		    case PARITY_CRC32_PR1_CCITT:
			printf("32-bit CRC-CCITT preset 1\n");
			break;
		    default:
			printf("unknown\n");
			break;
		}
	    }
	    break;

	case PC300_PROTO_MLPPP:
		printf("MLPPP\n");
		break;

	default:
	    printf("unknown\n");
	    break;
    }
    switch(conf->media) {
	case IF_IFACE_T1:
	case IF_IFACE_E1:
	    printf("- Clock:\t\t");
	    if (conf->phys_settings.clock_rate) {
		printf("master\n");
	    } else {
		printf("slave\n");
	    }
	    break;

	case IF_IFACE_V24:
	case IF_IFACE_V35:
	case IF_IFACE_X21:
	default:
	    printf("- Clock:\t\t");
	    switch (conf->phys_settings.clock_type) {
		case CLOCK_EXT:
		    printf("external\n");
		    break;
		case CLOCK_INT:
		    printf("internal (%u bps)\n", 
				(unsigned int)(conf->phys_settings.clock_rate));
		    break;
		case CLOCK_TXINT:
		    printf("internal TX clock (%u bps), external RX clock\n", 
				(unsigned int)(conf->phys_settings.clock_rate));
		    break;
		case CLOCK_TXFROMRX:
		    printf("TX clock generated from RX clock\n");
		    break;
		default:
		    printf("unknown\n");
		    break;
	    }
	    break;
    }
}

static void
pc300_set_if(char *cpc_name, pc300chconf_t *conf)
{
    struct ifreq ifr;

    strncpy(ifr.ifr_name, cpc_name, IFNAMSIZ);
    ifr.ifr_data = (caddr_t) conf;
    if (ioctl(skfd, SIOCSPC300CONF, &ifr) < 0) {
	printf("ioctl err\n");
	(void) close(skfd);
	exit (-1);
    }
}

#define	IN_RANGE(val, minval, maxval)	\
		(((val) >= (minval)) && ((val) <= (maxval)))
static int
pc300_set_tslots(pc300chconf_t *conf, char *tslot_str)
{
    int i, j, is_range, tsloti, tslotf;
    char tmpstr[10], tmpstr2[10];
    int ch_range = ((conf->media == IF_IFACE_T1) ? 24 : 31);

    conf->tslot_bitmap = 0;

    for (i = 0 , is_range = 0 ; tslot_str[i] != '\0' ; i+=j) {
	for (j = 0 ; tslot_str[i+j] != '\0' ; j++) {
	    if (tslot_str[i+j] == ',') {
		i++;
		break;
	    } else if (tslot_str[i+j] == '-') {
		tmpstr[j] = '\0';
		i += j;
		j = 0;
		is_range = 1;
	    } else if (is_range) {
		tmpstr2[j - 1] = tslot_str[i+j];
	    } else {
		tmpstr[j] = tslot_str[i+j];
	    }
	}
	if (is_range) {
	    is_range = 0;
	    tmpstr2[j - 1] = '\0';
	} else {
	    tmpstr[j] = '\0';
	    strcpy(tmpstr2, tmpstr);
	}
	for (tsloti = atoi(tmpstr), tslotf = atoi(tmpstr2) ;
	     tsloti <= tslotf ; 
	     tsloti++) {
	    if (IN_RANGE(tsloti, 1, ch_range)) {
		conf->tslot_bitmap |= 1 << (tsloti - 1);
	    } else {
		return (-1);
	    }
	}
    }
    return 0;
}

int main(int argc, char **argv)
{
    char *ifname;
    pc300conf_t pc300conf;
    pc300chconf_t *conf = &pc300conf.conf;
    struct ifreq ifr;
    int i;

    if (argc < 2) {
	printf("You must specify the interface (e.g. pc300cfg hdlc0).\n");
	exit(1);
    }

    ifname = argv[1];
    memset(&pc300conf, 0, sizeof(pc300conf_t));

    /* Create a channel to the NET kernel. */
    if ((skfd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
	printf("socket err\n");
	exit(1);
    }

    strncpy(ifr.ifr_name, ifname, IFNAMSIZ);
    ifr.ifr_data = (caddr_t) &pc300conf;
    if (ioctl(skfd, SIOCGPC300CONF, &ifr) < 0) {
	printf("ioctl err\n");
	(void) close(skfd);
	exit(-1);
    }
    
    if (argc < 3) {
	pc300_show_if(ifname, conf); 
    } else {
	for (i = 2 ; i < argc ; i += 2) {
	    if (strcmp(argv[i], "media") == 0) {
		if (strcmp(argv[i+1], "rs232") == 0) {
		    conf->media = IF_IFACE_V24;
		} else if (strcmp(argv[i+1], "v35") == 0) {
		    conf->media = IF_IFACE_V35;
		} else if (strcmp(argv[i+1], "x21") == 0) {
		    conf->media = IF_IFACE_X21;
		} else if (strcmp(argv[i+1], "t1") == 0) {
		    conf->media = IF_IFACE_T1;
		} else if (strcmp(argv[i+1], "e1") == 0) {
		    conf->media = IF_IFACE_E1;
		} else {
		    printf("Unknown media option: %s.\n", argv[i+1]);
		    exit(1);
		}
	    } else if (strcmp(argv[i], "proto") == 0) {
		if (strcmp(argv[i+1], "ppp") == 0) {
		    conf->proto = IF_PROTO_PPP;
		    conf->monitor = 0;
		} else if (strcmp(argv[i+1], "cisco") == 0) {
		    conf->proto = IF_PROTO_CISCO;
		    conf->monitor = 0;
		} else if (strcmp(argv[i+1], "fr") == 0) {
		    conf->proto = IF_PROTO_FR;
		    conf->monitor = 0;
		} else if (strcmp(argv[i+1], "monitor") == 0) {
		    conf->proto = IF_PROTO_HDLC;
		    conf->monitor = 1;
		} else if (strcmp(argv[i+1], "x25") == 0) {
		    conf->proto = IF_PROTO_X25;
		    conf->monitor = 0;
		} else 
		    if (strcmp(argv[i+1],"mlppp") == 0){
			conf->proto = PC300_PROTO_MLPPP;
			conf->monitor = 0;
		    } else 
			if (strcmp(argv[i+1],"mlppp-down") == 0){
			    conf->proto = 0xffff;
			    conf->monitor = 0;
			} else {
		    	    printf("Unknown proto option: %s.\n", argv[i+1]);
		    	    exit(1);
			}
	    } else if (strcmp(argv[i], "clock") == 0) {
		if (strcmp(argv[i+1], "int") == 0) {
		    if ((i + 2) < argc) {
			conf->phys_settings.clock_rate = atol(argv[i+2]);
			i++;
		    }
		    conf->phys_settings.clock_type = CLOCK_INT;
		} else if (strcmp(argv[i+1], "txint") == 0) {
		    if ((i + 2) < argc) {
			conf->phys_settings.clock_rate = atol(argv[i+2]);
			i++;
		    }
		    conf->phys_settings.clock_type = CLOCK_TXINT;
		} else if (strcmp(argv[i+1], "txfromrx") == 0) {
		    conf->phys_settings.clock_rate = 0;
		    conf->phys_settings.clock_type = CLOCK_TXFROMRX;
		} else { /* ext */
		    conf->phys_settings.clock_rate = 0;
		    conf->phys_settings.clock_type = CLOCK_EXT;
		}
	    } else if (strcmp(argv[i], "lcode") == 0) {
		if (strcmp(argv[i+1], "ami") == 0) {
		    conf->lcode = PC300_LC_AMI;
		} else if (strcmp(argv[i+1], "b8zs") == 0) {
		    conf->lcode = PC300_LC_B8ZS;
		} else if (strcmp(argv[i+1], "nrz") == 0) {
		    conf->lcode = PC300_LC_NRZ;
		} else if (strcmp(argv[i+1], "hdb3") == 0) {
		    conf->lcode = PC300_LC_HDB3;
		} else {
		    printf("Unknown lcode option: %s.\n", argv[i+1]);
		    exit(1);
		}
	    } else if (strcmp(argv[i], "frame") == 0) {
		if (strcmp(argv[i+1], "d4") == 0) {
		    conf->fr_mode = PC300_FR_D4;
		} else if (strcmp(argv[i+1], "esf") == 0) {
		    conf->fr_mode = PC300_FR_ESF;
		} else if (strcmp(argv[i+1], "crc4") == 0) {
		    conf->fr_mode = PC300_FR_MF_CRC4;
		} else if (strcmp(argv[i+1], "non-crc4") == 0) {
		    conf->fr_mode = PC300_FR_MF_NON_CRC4;
		} else if (strcmp(argv[i+1], "unframed") == 0) {
		    conf->fr_mode = PC300_FR_UNFRAMED;
		} else {
		    printf("Unknown frame option: %s.\n", argv[i+1]);
		    exit(1);
		}
	    } else if (strcmp(argv[i], "lbo") == 0) {
		if (strcmp(argv[i+1], "0") == 0) {
		    conf->lbo = PC300_LBO_0_DB;
		} else if (strcmp(argv[i+1], "7.5") == 0) {
		    conf->lbo = PC300_LBO_7_5_DB;
		} else if (strcmp(argv[i+1], "15") == 0) {
		    conf->lbo = PC300_LBO_15_DB;
		} else if (strcmp(argv[i+1], "22.5") == 0) {
		    conf->lbo = PC300_LBO_22_5_DB;
		} else {
		    printf("Unknown lbo option: %s.\n", argv[i+1]);
		    exit(1);
		}
	    } else if (strcmp(argv[i], "rx_sens") == 0) {
		if (strcmp(argv[i+1], "sh") == 0) {
		    conf->rx_sens = PC300_RX_SENS_SH;
		} else if (strcmp(argv[i+1], "lh") == 0) {
		    conf->rx_sens = PC300_RX_SENS_LH;
		} else {
		    printf("Unknown rx_sens option: %s.\n", argv[i+1]);
		    exit(1);
		}
	    } else if (strcmp(argv[i], "active_ch") == 0) {
		if (strcmp(argv[i+1], "all") == 0) {
		    conf->tslot_bitmap = 0xffffffffUL;
		} else if (pc300_set_tslots(conf, argv[i+1]) != 0) {
		    printf("Invalid channel allocation: %s.\n", argv[i+1]);
		    exit(1);
		}
	    } else {
		printf("Unknown option: %s.\n", argv[i]);
		exit(1);
	    }
	}
	pc300_set_if(ifname, conf);
    }
	
    (void) close(skfd);
    return 0;
}
